@@ -9,8 +9,8 @@ extern "C" {
 #endif
 
 #include "jdupes.h"
-extern void printmatches(file_t * restrict files);
-extern void printunique(file_t * restrict files);
+void printmatches(file_t * restrict files);
+void printunique(file_t *files);
 
 #ifdef __cplusplus
 }
