@@ -50,6 +50,9 @@
 #endif
 #include "filehash.h"
 #include "filestat.h"
+#ifndef NO_HASHDB
+ #include "hashdb.h"
+#endif
 #include "helptext.h"
 #include "loaddir.h"
 #include "match.h"
@@ -140,6 +143,13 @@ uintmax_t comparisons = 0;
 /* File tree head */
 static filetree_t *checktree = NULL;
 
+/* Hash algorithm (see filehash.h) */
+#ifdef USE_JODY_HASH
+int hash_algo = HASH_ALGO_JODYHASH64;
+#else
+int hash_algo = HASH_ALGO_XXHASH2_64;
+#endif
+
 /* Directory/file parameter position counter */
 unsigned int user_item_count = 1;
 
@@ -188,42 +198,39 @@ int main(int argc, char **argv)
   static struct utsname utsname;
  #endif /* __linux__ */
 #endif
+#ifndef NO_HASHDB
+  char *hashdb_name = NULL;
+  int hdblen;
+  int64_t hdbsize;
+  uint64_t hdbout;
+#endif
 
 #ifndef NO_GETOPT_LONG
   static const struct option long_options[] =
   {
     { "loud", 0, 0, '@' },
-    { "printnull", 0, 0, '0' }, //LEGACY
     { "print-null", 0, 0, '0' },
     { "one-file-system", 0, 0, '1' },
     { "", 0, 0, '9' },
-    { "nohidden", 0, 0, 'A' }, //LEGACY
     { "no-hidden", 0, 0, 'A' },
     { "dedupe", 0, 0, 'B' },
-    { "chunksize", 1, 0, 'C' }, //LEGACY
     { "chunk-size", 1, 0, 'C' },
     { "debug", 0, 0, 'D' },
     { "delete", 0, 0, 'd' },
-    { "error-on-dupe", 0, 0, 'E' },
-    { "omitfirst", 0, 0, 'f' }, //LEGACY
+    { "error-on-dupe", 0, 0, 'e' },
+    { "ext-option", 0, 0, 'E' },
     { "omit-first", 0, 0, 'f' },
-    { "hardlinks", 0, 0, 'H' }, //LEGACY
     { "hard-links", 0, 0, 'H' },
     { "help", 0, 0, 'h' },
     { "isolate", 0, 0, 'I' },
     { "reverse", 0, 0, 'i' },
     { "json", 0, 0, 'j' },
 /*    { "skip-hash", 0, 0, 'K' }, */
-    { "linkhard", 0, 0, 'L' }, //LEGACY
     { "link-hard", 0, 0, 'L' },
-    { "linksoft", 0, 0, 'l' }, //LEGACY
     { "link-soft", 0, 0, 'l' },
-    { "printwithsummary", 0, 0, 'M'}, //LEGACY
     { "print-summarize", 0, 0, 'M'},
     { "summarize", 0, 0, 'm'},
-    { "noprompt", 0, 0, 'N' }, //LEGACY
     { "no-prompt", 0, 0, 'N' },
-    { "paramorder", 0, 0, 'O' }, //LEGACY
     { "param-order", 0, 0, 'O' },
     { "order", 1, 0, 'o' },
     { "print", 1, 0, 'P' },
@@ -235,18 +242,13 @@ int main(int argc, char **argv)
     { "size", 0, 0, 'S' },
     { "symlinks", 0, 0, 's' },
     { "partial-only", 0, 0, 'T' },
-    { "nochangecheck", 0, 0, 't' }, //LEGACY
     { "no-change-check", 0, 0, 't' },
-    { "notravcheck", 0, 0, 'U' }, //LEGACY
     { "no-trav-check", 0, 0, 'U' },
-    { "printunique", 0, 0, 'u' }, //LEGACY
     { "print-unique", 0, 0, 'u' },
     { "version", 0, 0, 'v' },
-    { "extfilter", 1, 0, 'X' }, //LEGACY
     { "ext-filter", 1, 0, 'X' },
-    { "softabort", 0, 0, 'Z' }, //LEGACY
+    { "hash-db", 1, 0, 'y' },
     { "soft-abort", 0, 0, 'Z' },
-    { "zeromatch", 0, 0, 'z' }, //LEGACY
     { "zero-match", 0, 0, 'z' },
     { NULL, 0, 0, 0 }
   };
@@ -255,7 +257,7 @@ int main(int argc, char **argv)
  #define GETOPT getopt
 #endif
 
-#define GETOPT_STRING "@019ABC:DdEfHhIijKLlMmNnOo:P:pQqRrSsTtUuVvX:Zz"
+#define GETOPT_STRING "@019ABC:DdEefHhIijKLlMmNnOo:P:pQqRrSsTtUuVvX:y:Zz"
 
   /* Verify libjodycode compatibility before going further */
   if (libjodycode_version_check(1, 0) != 0) {
@@ -332,6 +334,28 @@ int main(int argc, char **argv)
     case 'A':
       SETFLAG(flags, F_EXCLUDEHIDDEN);
       break;
+#ifdef ENABLE_DEDUPE
+    case 'B':
+#ifdef __linux__
+      /* Refuse to dedupe on 2.x kernels; they could damage user data */
+      if (uname(&utsname)) {
+        fprintf(stderr, "Failed to get kernel version! Aborting.\n");
+        exit(EXIT_FAILURE);
+      }
+      LOUD(fprintf(stderr, "dedupefiles: uname got release '%s'\n", utsname.release));
+      if (*(utsname.release) == '2' && *(utsname.release + 1) == '.') {
+        fprintf(stderr, "Refusing to dedupe on a 2.x kernel; data loss could occur. Aborting.\n");
+        exit(EXIT_FAILURE);
+      }
+      /* Kernel-level dedupe will do the byte-for-byte check itself */
+      if (!ISFLAG(flags, F_PARTIALONLY)) SETFLAG(flags, F_QUICKCOMPARE);
+#endif /* __linux__ */
+      SETFLAG(a_flags, FA_DEDUPEFILES);
+      /* It is completely useless to dedupe zero-length extents */
+      CLEARFLAG(flags, F_INCLUDEEMPTY);
+      LOUD(fprintf(stderr, "opt: CoW/block-level deduplication enabled (--dedupe)\n");)
+      break;
+#endif /* ENABLE_DEDUPE */
 #ifndef NO_CHUNKSIZE
     case 'C':
       manual_chunk_size = (strtol(optarg, NULL, 10) & 0x0ffffffcL) << 10;  /* Align to 4K sizes */
@@ -359,6 +383,11 @@ int main(int argc, char **argv)
       break;
 #ifndef NO_ERRORONDUPE
     case 'E':
+      fprintf(stderr, "The -E option has been moved to -e as threatened in 1.26.1!\n");
+      fprintf(stderr, "Fix whatever used -E and try again. This is not a bug. Exiting.\n");
+      exit(EXIT_FAILURE);
+      break;
+    case 'e':
       SETFLAG(a_flags, FA_ERRORONDUPE);
       break;
 #endif /* NO_ERRORONDUPE */
@@ -368,7 +397,7 @@ int main(int argc, char **argv)
       break;
     case 'h':
       help_text();
-      exit(EXIT_FAILURE);
+      exit(EXIT_SUCCESS);
 #ifndef NO_HARDLINKS
     case 'H':
       SETFLAG(flags, F_CONSIDERHARDLINKS);
@@ -422,6 +451,18 @@ int main(int argc, char **argv)
       LOUD(fprintf(stderr, "opt: delete files without prompting (--noprompt)\n");)
       break;
 #endif /* NO_DELETE */
+    case 'o':
+#ifndef NO_MTIME  /* Remove if new order types are added! */
+      if (!jc_strncaseeq("name", optarg, 5)) {
+        ordertype = ORDER_NAME;
+      } else if (!jc_strncaseeq("time", optarg, 5)) {
+        ordertype = ORDER_TIME;
+      } else {
+        fprintf(stderr, "invalid value for --order: '%s'\n", optarg);
+        exit(EXIT_FAILURE);
+      }
+#endif /* NO_MTIME */
+      break;
     case 'p':
       SETFLAG(flags, F_PERMISSIONS);
       LOUD(fprintf(stderr, "opt: permissions must also match (--permissions)\n");)
@@ -441,7 +482,6 @@ int main(int argc, char **argv)
       break;
     case 'Q':
       SETFLAG(flags, F_QUICKCOMPARE);
-      fprintf(stderr, "\nBIG FAT WARNING: -Q/--quick MAY BE DANGEROUS! Read the manual!\n\n");
       LOUD(fprintf(stderr, "opt: byte-for-byte safety check disabled (--quick)\n");)
       break;
     case 'r':
@@ -473,6 +513,10 @@ int main(int argc, char **argv)
       SETFLAG(flags, F_NOTRAVCHECK);
       LOUD(fprintf(stderr, "opt: double-traversal safety check disabled (--no-trav-check)\n");)
       break;
+    case 'v':
+    case 'V':
+      version_text(0);
+      exit(EXIT_SUCCESS);
 #ifndef NO_SYMLINKS
     case 'l':
       SETFLAG(a_flags, FA_MAKESYMLINKS);
@@ -492,6 +536,23 @@ int main(int argc, char **argv)
       add_extfilter(optarg);
       break;
 #endif /* NO_EXTFILTER */
+#ifndef NO_HASHDB
+    case 'y':
+      SETFLAG(flags, F_HASHDB);
+      LOUD(fprintf(stderr, "opt: use a hash database (--hash-db)\n");)
+      fprintf(stderr, "\nWARNING: THE HASH DATABASE FEATURE IS UNDER HEAVY DEVELOPMENT! It functions\n");
+      fprintf(stderr,   "         but there are LOTS OF QUIRKS. The behavior is not fully documented\n");
+      fprintf(stderr,   "         yet and basic 'smarts' have not been implemented. USE THIS FEATURE\n");
+      fprintf(stderr,   "         AT YOUR OWN RISK. Report hashdb issues to jody@jodybruchon.com\n\n");
+      hdbsize = 0;
+      hdblen = strlen(optarg) + 1;
+      if (hdblen < 24) hdblen = 24;
+      hashdb_name = (char *)malloc(hdblen);
+      if (hashdb_name == NULL) jc_nullptr("hashdb");
+      if (strcmp(optarg, ".") == 0) strcpy(hashdb_name, "jdupes_hashdb.txt");
+      else strcpy(hashdb_name, optarg);
+      break;
+#endif /* NO_HASHDB */
     case 'z':
       SETFLAG(flags, F_INCLUDEEMPTY);
       LOUD(fprintf(stderr, "opt: zero-length files count as matches (--zero-match)\n");)
@@ -506,47 +567,6 @@ int main(int argc, char **argv)
 #endif
       LOUD(fprintf(stderr, "opt: loud debugging enabled, hope you can handle it (--loud)\n");)
       break;
-    case 'v':
-    case 'V':
-      version_text(0);
-      exit(EXIT_SUCCESS);
-    case 'o':
-#ifndef NO_MTIME  /* Remove if new order types are added! */
-      if (!jc_strncaseeq("name", optarg, 5)) {
-        ordertype = ORDER_NAME;
-      } else if (!jc_strncaseeq("time", optarg, 5)) {
-        ordertype = ORDER_TIME;
-      } else {
-        fprintf(stderr, "invalid value for --order: '%s'\n", optarg);
-        exit(EXIT_FAILURE);
-      }
-#endif /* NO_MTIME */
-      break;
-    case 'B':
-#ifdef ENABLE_DEDUPE
-#ifdef __linux__
-      /* Refuse to dedupe on 2.x kernels; they could damage user data */
-      if (uname(&utsname)) {
-        fprintf(stderr, "Failed to get kernel version! Aborting.\n");
-        exit(EXIT_FAILURE);
-      }
-      LOUD(fprintf(stderr, "dedupefiles: uname got release '%s'\n", utsname.release));
-      if (*(utsname.release) == '2' && *(utsname.release + 1) == '.') {
-        fprintf(stderr, "Refusing to dedupe on a 2.x kernel; data loss could occur. Aborting.\n");
-        exit(EXIT_FAILURE);
-      }
-      /* Kernel-level dedupe will do the byte-for-byte check itself */
-      if (!ISFLAG(flags, F_PARTIALONLY)) SETFLAG(flags, F_QUICKCOMPARE);
-#endif /* __linux__ */
-      SETFLAG(a_flags, FA_DEDUPEFILES);
-      /* It is completely useless to dedupe zero-length extents */
-      CLEARFLAG(flags, F_INCLUDEEMPTY);
-#else /* ENABLE_DEDUPE */
-      fprintf(stderr, "This program was built without dedupe support\n");
-      exit(EXIT_FAILURE);
-#endif /* ENABLE_DEDUPE */
-      LOUD(fprintf(stderr, "opt: CoW/block-level deduplication enabled (--dedupe)\n");)
-      break;
 
     default:
       if (opt != '?') fprintf(stderr, "Sorry, using '-%c' is not supported in this build.\n", opt);
@@ -629,6 +649,14 @@ skip_partialonly_noise:
   /* Catch CTRL-C */
   signal(SIGINT, catch_interrupt);
 
+#ifndef NO_HASHDB
+  if (ISFLAG(flags, F_HASHDB)) {
+    hdbsize = load_hash_database(hashdb_name);
+    if (hdbsize < 0) goto error_load_hashdb;
+    if (hdbsize > 0 && !ISFLAG(flags, F_HIDEPROGRESS)) fprintf(stderr, "%" PRId64 " entries loaded.\n", hdbsize);
+  }
+#endif /* NO_HASHDB */
+
   /* Progress indicator every second */
   if (!ISFLAG(flags, F_HIDEPROGRESS)) {
     jc_start_alarm(1, 1);
@@ -728,11 +756,11 @@ skip_partialonly_noise:
              ISFLAG(flags, F_QUICKCOMPARE)
           || ISFLAG(flags, F_PARTIALONLY)
 #ifndef NO_HARDLINKS
-	  || (ISFLAG(flags, F_CONSIDERHARDLINKS)
+          || (ISFLAG(flags, F_CONSIDERHARDLINKS)
           &&  (curfile->inode == (*match)->inode)
           &&  (curfile->device == (*match)->device))
 #endif
-	  ) {
+          ) {
         LOUD(fprintf(stderr, "MAIN: notice: hard linked, quick, or partial-only match (-H/-Q/-T)\n"));
 #ifndef NO_MTIME
         registerpair(match, curfile, (ordertype == ORDER_TIME) ? sort_pairs_by_mtime : sort_pairs_by_filename);
@@ -830,15 +858,26 @@ skip_file_scan:
     summarizematches(files);
   }
 
+#ifndef NO_HASHDB
+  if (ISFLAG(flags, F_HASHDB)) {
+    hdbout = save_hash_database(hashdb_name, 1);
+    if (!ISFLAG(flags, F_HIDEPROGRESS)) {
+      if (hdbout > 0) fprintf(stderr, "Wrote %" PRIu64 " entries to the hash database\n", hdbout);
+      else fprintf(stderr, "Hash database is OK (no changes)\n");
+    }
+  }
+  if (hashdb_name != NULL) free(hashdb_name);
+#endif
+
 #ifdef DEBUG
 skip_all_scan_code:
 #endif
 
 #ifdef DEBUG
   if (ISFLAG(flags, F_DEBUG)) {
-    fprintf(stderr, "\n%d partial (+%d small) -> %d full hash -> %d full (%d partial elim) (%d hash%u fail)\n",
-        partial_hash, small_file, full_hash, partial_to_full,
-        partial_elim, hash_fail, (unsigned int)sizeof(jdupes_hash_t)*8);
+    fprintf(stderr, "\n%d partial(%uKiB) (+%d small) -> %d full hash -> %d full (%d partial elim) (%d hash%u fail)\n",
+        partial_hash, PARTIAL_HASH_SIZE >> 10, small_file, full_hash, partial_to_full,
+        partial_elim, hash_fail, (unsigned int)sizeof(uint64_t)*8);
     fprintf(stderr, "%" PRIuMAX " total files, %" PRIuMAX " comparisons\n", filecount, comparisons);
  #ifndef NO_CHUNKSIZE
     if (manual_chunk_size > 0) fprintf(stderr, "I/O chunk size: %ld KiB (manually set)\n", manual_chunk_size >> 10);
@@ -864,6 +903,11 @@ skip_all_scan_code:
 error_optarg:
   fprintf(stderr, "error: option '%c' requires an argument\n", opt);
   exit(EXIT_FAILURE);
+#ifndef NO_HASHDB
+error_load_hashdb:
+  free(hashdb_name);
+  exit(EXIT_FAILURE);
+#endif
 interrupt_exit:
   fprintf(stderr, "%s", s_interrupt);
   exit(EXIT_FAILURE);
