@@ -27,18 +27,6 @@ extern "C" {
 #include <stdint.h>
 #include <sys/types.h>
 
-#ifndef USE_JODY_HASH
-#include "xxhash.h"
-#endif /* USE_JODY_HASH */
-
-/* Set hash type (change this if swapping in a different hash function) */
-#ifdef USE_JODY_HASH
- #include "libjodycode.h"
- typedef jodyhash_t jdupes_hash_t;
-#else
- typedef XXH64_hash_t jdupes_hash_t;
-#endif /* USE_JODY_HASH */
-
 /* Some types are different on Windows */
 #if defined _WIN32 || defined __MINGW32__
  typedef uint64_t jdupes_ino_t;
@@ -85,7 +73,7 @@ extern "C" {
  * Windows + Unicode needs a lot more space than UTF-8 in Linux/Mac OS X
  */
 #ifndef PATHBUF_SIZE
-#define PATHBUF_SIZE 4096
+ #define PATHBUF_SIZE 4096
 #endif
 /* Complain if PATHBUF_SIZE is too small */
 #if PATHBUF_SIZE < PATH_MAX
@@ -153,6 +141,8 @@ extern uintmax_t comparisons;
 /* Compare two hashes like memcmp() */
 #define HASH_COMPARE(a,b) ((a > b) ? 1:((a == b) ? 0:-1))
 
+/* Extend an allocation length to the next 64-bit (8-byte) boundary */
+#define EXTEND64(a) ((a & 0x7) > 0 ? ((a & (~0x7)) + 8) : a)
 
 /* Behavior modification flags */
 extern uint64_t flags, a_flags, p_flags;
@@ -177,6 +167,7 @@ extern uint64_t flags, a_flags, p_flags;
 #define F_NOTRAVCHECK		(1ULL << 18)
 #define F_SKIPHASH		(1ULL << 19)
 #define F_BENCHMARKSTOP		(1ULL << 29)
+#define F_HASHDB		(1ULL << 30)
 
 #define F_LOUD			(1ULL << 62)
 #define F_DEBUG			(1ULL << 63)
@@ -202,6 +193,7 @@ extern uint64_t flags, a_flags, p_flags;
 #define FF_HAS_DUPES		(1U << 3)
 #define FF_IS_SYMLINK		(1U << 4)
 #define FF_NOT_UNIQUE		(1U << 5)
+#define FF_HASHDB_DIRTY		(1U << 5)
 
 /* Extra print flags */
 #define PF_PARTIAL		(1U << 0)
@@ -217,31 +209,21 @@ typedef enum {
  #define PARTIAL_HASH_SIZE 4096
 #endif
 
-/* Maximum path buffer size to use; must be large enough for a path plus
- * any work that might be done to the array it's stored in. PATH_MAX is
- * not always true. Read this article on the false promises of PATH_MAX:
- * http://insanecoding.blogspot.com/2007/11/pathmax-simply-isnt.html
- * Windows + Unicode needs a lot more space than UTF-8 in Linux/Mac OS X
- */
-#ifndef PATHBUF_SIZE
-#define PATHBUF_SIZE 4096
-#endif
-
 /* Per-file information */
 typedef struct _file {
   struct _file *duplicates;
   struct _file *next;
   char *d_name;
-  uint32_t flags;  /* Status flags */
-  jdupes_mode_t mode;
-  off_t size;
-  dev_t device;
+  uint64_t filehash_partial;
+  uint64_t filehash;
   jdupes_ino_t inode;
-  jdupes_hash_t filehash_partial;
-  jdupes_hash_t filehash;
+  off_t size;
 #ifndef NO_MTIME
   time_t mtime;
 #endif
+  dev_t device;
+  uint32_t flags;  /* Status flags */
+  jdupes_mode_t mode;
 #ifndef NO_ATIME
   time_t atime;
 #endif
@@ -279,6 +261,7 @@ typedef struct _filetree {
 /* Progress indicator variables */
 extern uintmax_t filecount, progress, item_progress, dupecount;
 
+extern int hash_algo;
 extern unsigned int user_item_count;
 extern int sort_direction;
 extern char tempname[];
